<?php
if(preg_match('#' . basename(__FILE__) . '#', $_SERVER['PHP_SELF'])) { die('You are not allowed to call this page directly.'); }

/**
 * Add default options and capabilties 
 */

function nginxm_install () {
	
   	global $wp_roles, $nginxm;
   	
	// Check for capability
	if ( !current_user_can('activate_plugins') ) 
		return;
	
	// Set the capabilities for the administrator
	$role = get_role('administrator');
	// We need this role, no other chance
	if ( empty($role) ) {
		update_site_option( "nginxm_init_check", __('Sorry, Nginx Manager works only with a role called administrator',"nginxm") );
		return;
	}
   	
	$role->add_cap('Nginx Manager | Config');
	$role->add_cap('Nginx Manager | Purge cache');
	
	$nginxm_global_options = get_site_option('nginxm_global_options');
	
	// Set the GLOBAL default settings, if we didn't upgrade
	if ( empty( $nginxm_global_options ) ) {
		
		// Cookie and cache timelife
		$nginxm_global_options['cookie_regexp']         = '/wordpress_(?!test_cookie)|comment_author|wp-postpass/';
		$nginxm_global_options['cache_ttl']             = '600';
		
		// Log settings
		$nginxm_global_options['log_level']             = 'INFO';
		$nginxm_global_options['log_filesize']          = 5;
		
		// Set plugin version, @since Nginx Manager 1.2
		$nginxm_global_options['current_db_version']    = $nginxm->db_version;
		
		update_site_option("nginxm_global_options", $nginxm_global_options);
		
	}
	
	// check if multisite
	if (is_multisite()) {
		
		// multisite installation
		
		// Set the default settings for each blog, if we didn't upgrade
		$blogs = get_blogs_of_user(true);
	
		foreach ($blogs as $b) {
			
			$nginxm_options = get_blog_option($b->userblog_id, 'nginxm_options');
			
			// Set the LOCAL default settings, if we didn't upgrade
			if ( empty( $nginxm_options ) ) {
				
				// Options for homepage, personal URLs and an array for capture URL
				$nginxm_options['automatically_purge_homepage']                 = 1;
				$nginxm_options['automatically_purge_purls']                    = 1;
				$nginxm_options['purgeable_url']                                = array();
				
				// Options "When a post/page is modified"
				$nginxm_options['automatically_purge_page']                     = 1;
				$nginxm_options['automatically_purge_page_archive']             = 1;
				$nginxm_options['automatically_purge_page_custom_taxa']         = 1;
				
				// Options "When a new comment is published"
				$nginxm_options['automatically_purge_comment_page']             = 1;
				$nginxm_options['automatically_purge_comment_archive']          = 1;
				$nginxm_options['automatically_purge_comment_custom_taxa']      = 1;
				
				update_blog_option($b->userblog_id, "nginxm_options", $nginxm_options);
				
			}
				
		}
	} else {
		
		// it's a single installation
		$nginxm_options = get_option('nginxm_options');
		
		// Set the LOCAL default settings, if we didn't upgrade
		if ( empty( $nginxm_options ) ) {
			
			// Options for homepage, personal URLs and an array for capture URL
			$nginxm_options['automatically_purge_homepage']                 = 1;
			$nginxm_options['automatically_purge_purls']                    = 1;
			$nginxm_options['purgeable_url']                                = array();
			
			// Options "When a post/page is modified"
			$nginxm_options['automatically_purge_page']                     = 1;
			$nginxm_options['automatically_purge_page_archive']             = 1;
			$nginxm_options['automatically_purge_page_custom_taxa']         = 1;
			
			// Options "When a new comment is published"
			$nginxm_options['automatically_purge_comment_page']             = 1;
			$nginxm_options['automatically_purge_comment_archive']          = 1;
			$nginxm_options['automatically_purge_comment_custom_taxa']      = 1;
			
			update_option("nginxm_options", $nginxm_options);
			
		}
		
	}
	
	// setup cron event
	wp_schedule_event(time(), 'daily', 'nm_check_log_file_size_daily');
	
}

/**
 * Remove all options and capabilities
 */

function nginxm_uninstall() {
	
	global $nginxm, $nginxmNginx;
	
	// unschedule events
	wp_clear_scheduled_hook('nm_check_log_file_size_daily');
	
	// then remove all options
	delete_site_option( 'nginxm_global_options' );
	
	// check if multisite
	if (is_multisite()) {
		
		// multisite installation
		$blogs = get_blogs_of_user(true);
		foreach ($blogs as $b) {
			delete_blog_option( $b->userblog_id, 'nginxm_options' );
		}
		
	} else {
		// single installation
		delete_option( 'nginxm_options' );
	}

	// now remove the capability
	nginxm_remove_capability('Nginx Manager | Config');
	nginxm_remove_capability('Nginx Manager | Purge cache');	
	
}

/**
 * Deregister a capability from all classic roles
 */

function nginxm_remove_capability($capability){
	// this function remove the $capability only from the classic roles
	$check_order = array("subscriber", "contributor", "author", "editor", "administrator");

	foreach ($check_order as $role) {
		$role = get_role($role);
		$role->remove_cap($capability) ;
	}

}

?>