<?php
/*  Copyright 2007    Simone Fumagalli

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

if (!class_exists('nginxmNginx')) {
	
	class nginxmNginx {
		
		function correctExpires($http_headers) {
			
			global $nginxm;
			
			$cache_ttl = 0;
			
			// If it's an HTML document or a feed
		    if ( ( strpos($http_headers['Content-Type'], "text/html") !== false ) || ( strpos($http_headers['Content-Type'], "text/xml") !== false ) ) {
		       $cache_ttl = $nginxm->global_options['cache_ttl'];
		    } 	
			
			// If the user is logged in
		    foreach($_COOKIE as $key => $value){
		        if ( preg_match($nginxm->global_options['cookie_regexp'], $key) ) {
		            $cache_ttl = 0;
		        }
		    }
		    
		    $http_headers['X-Accel-Expires'] = $cache_ttl;
		    
		    if ($cache_ttl == 0) {

		    	$http_headers['X-NGINX-CACHED']  = "NO";
		    	
		    } else {

		    	$http_headers['X-NGINX-CACHED']  	= "YES - ".$cache_ttl." secs";
		    	$http_headers['X-NGINX-CACHED-AT']  = date('c');
		    	
		    }
		    
		    return $http_headers;
		
		}

		function purgePost($_ID) {
			
			global $nginxm, $blog_id;
			
			$this->log( "Function purgePost BEGIN ===" );
			
			switch ( current_filter() ) {
				case 'comment_post':
					$_comment_ID = $_ID;
					$_comment = get_comment($_ID);
					$_ID = $_comment->comment_post_ID;
					$this->log( "New comment published/edited (id $_comment_ID) on post ($_ID) on blog '".addslashes(get_bloginfo('name'))."' ($blog_id)" );
					break;
				case 'publish_post':
					$this->log( "New post published/edited (id $_ID) on blog '".addslashes(get_bloginfo('name'))."' ($blog_id)" );
					break;
				case 'publish_page':
					$this->log( "New page published/edited (id $_ID) on blog '".addslashes(get_bloginfo('name'))."' ($blog_id)" );
					break;
			}
			
			/* Homepage */
			if ($nginxm->options['automatically_purge_homepage'] == 1) {
				
				$homepage_url = trailingslashit(get_option('siteurl'));
				
				$this->log( "Purging homepage '$homepage_url'" );
				$this->purgeUrl($homepage_url);
				
			}
			
			/* Personal URLs */
			if ($nginxm->options['automatically_purge_purls'] == 1) {
				
				$this->log( "Purging personal urls" );
				
				if (isset($nginxm->options['purgeable_url']['urls'])) {
					foreach ($nginxm->options['purgeable_url']['urls'] as $u) {
						$this->purgeUrl($u, false);
					}
				} else {
					$this->log( "No personal urls available" );
				}
			}
			
			/* Post or comment */
			if ( current_filter() == 'comment_post' ) {
				
				/* When a new comment is published */
				$this->_purge_by_options($_ID, $blog_id, $nginxm->options['automatically_purge_comment_page'], $nginxm->options['automatically_purge_comment_archive'], $nginxm->options['automatically_purge_comment_custom_taxa']);
				
			} else {
				
				/* When a post/page is modified */
				$this->_purge_by_options($_ID, $blog_id, $nginxm->options['automatically_purge_page'], $nginxm->options['automatically_purge_page_archive'], $nginxm->options['automatically_purge_page_custom_taxa']);
			}
			
			$this->log( "Function purgePost END ^^^" );
			
		}
		
		
		private function _purge_by_options($_post_ID, $blog_id, $_purge_page, $_purge_archive, $_purge_custom_taxa) {
			
			global $nginxm;
			
			$_post_type = get_post_type( $_post_ID );
			
			/* Purge cache for the related page/post */
			if ($_purge_page) {
				
				if ( $_post_type == 'post' || $_post_type == 'page' ) {
					$this->log( "Purging $_post_type (id $_post_ID, blog id $blog_id)" );
				} else {
					$this->log( "Purging custom post type '$_post_type' (id $_post_ID, blog id $blog_id)" );
				}
				
				$this->purgeUrl( get_permalink( $_post_ID ) );
			}
			
			/* Purge cache for archive related to the page (author, category, date) */
			if ($_purge_archive) {
				
				/* date */
				if ( $_post_type == 'post' ) {
					
					$this->log( "Purging date" );
					
					$day 	= get_the_time('d', $_post_ID);
					$month 	= get_the_time('m', $_post_ID);
					$year 	= get_the_time('Y', $_post_ID);
					
					if ( $year ) {
						$this->purgeUrl( get_year_link( $year ) );
						if ( $month ) {
							$this->purgeUrl( get_month_link( $year, $month ) );
							if ( $day )
								$this->purgeUrl( get_day_link( $year, $month, $day ) );
						}
					}
				}
				
				/* category */
				$this->log( "Purging category archives" );
				
				if ( $categories = wp_get_post_categories( $_post_ID ) ) {
					foreach ( $categories as $category_id ) {
						$this->log( "Purging category ".$category_id );
						$this->purgeUrl( get_category_link( $category_id ) );
					}
				}
				
				/* tags, @since Nginx Manager 1.2 */
				$this->log( "Purging tag archives" );
				
				if ( $tags = get_the_tags( $_post_ID ) ) {
					foreach ( $tags as $tag ) {
						$this->log( "Purging tag ".$tag->term_id );
						$this->purgeUrl( get_tag_link( $tag->term_id ) );
					}
				}
				
				/* author */
				$this->log( "Purging author archive" );
				
				if ( $author_id = get_post($_post_ID)->post_author )
					$this->purgeUrl( get_author_posts_url( $author_id ) );
				
			}
			
			/* Purge related custom terms, @since Nginx Manager 1.2 */
			if ( $_purge_custom_taxa ) {
				$this->log( "Purging custom taxonomies related" );
				if ( $custom_taxonomies = get_taxonomies( array( 'public' => true, '_builtin' => false ) ) ) {
					foreach ( $custom_taxonomies as $taxon ) {
						// Extra check to get rid of the standard taxonomies
						if ( !in_array( $taxon, array( 'category', 'post_tag', 'link_category' ) ) ) {
							if ( $terms = get_the_terms( $_post_ID, $taxon ) ) {
								foreach ( $terms as $term ) {
									$this->purgeUrl( get_term_link( $term, $taxon ) );
								}
							}
						} else {
							$this->log( "Your built-in taxonomy '".$taxon."' has param '_builtin' set to false.", "WARNING" );
						}
					}
				}
			}
			
		}
		
		
		function purgeUrl($url, $feed = true) {
			
			$this->log( "Purging URL | ".$url );
			
			$parse 		= parse_url($url);
			
			$this->_do_remote_get($parse['scheme'].'://'.$parse['host'].'/purge'.$parse['path']);
			
			if ($feed) {
				$feed_string = (substr($parse['path'], -1) != '/') ? "/feed/" : "feed/";
				$this->_do_remote_get($parse['scheme'].'://'.$parse['host'].'/purge'.$parse['path'].$feed_string);
			}
			
		}
		
		private function _do_remote_get($url) {
			
			$response = wp_remote_get($url);
			
			if( is_wp_error( $response ) ) {
				$_errors_str = implode(" - ",$response->get_error_messages());
				$this->log( "Error while purging URL. ".$_errors_str, "ERROR" );
			} else {
				if ( $response['response']['code'] ) {
					switch ( $response['response']['code'] ) {
						case 200:
							$this->log( $url." *** PURGED ***" );
							break;
						case 404:
							$this->log( $url." is currently not cached" );
							break;
						default:
							$this->log( $url." not found (".$response['response']['code'].")", "WARNING" );
					}
				}
		    }			
			
		}
		
		function checkHttpConnection() {
			
			$purgeURL = plugins_url("nginx-manager/check-proxy.php");
			$response = wp_remote_get($purgeURL);
			
			if( !is_wp_error( $response ) && ($response['body'] == 'HTTP Connection OK')) {
				return "OK";
			}
			
			return "KO";
			
		}
		
		function log( $msg, $level='INFO' ) {
			
			global $nginxm;
			
			$log_levels = array("INFO" => 0, "WARNING" => 1, "ERROR" => 2, "NONE" => 3);
			
			if ($log_levels[$level] >= $log_levels[$nginxm->global_options['log_level']]) {
				if ($fp = fopen(NGINXM_ABSPATH .'log/current.log',"a+")) {
					fwrite($fp, "\n".gmdate("Y-m-d H:i:s ")." | ".$level." | ".$msg);
					fclose($fp);
				}
			}
			
			return true;
			
		}
		
		function checkAndTruncateLogFile() {
			
			global $nginxm;
			
			$maxSizeAllowed = (is_numeric($nginxm->global_options['log_filesize'])) ? $nginxm->global_options['log_filesize']*1048576 : 5242880;
			
			$fileSize = filesize(NGINXM_ABSPATH .'log/current.log');
			
			if ($fileSize > $maxSizeAllowed) {
				
				$offset = $fileSize - $maxSizeAllowed;
				
				if ($file_content = file_get_contents(NGINXM_ABSPATH .'log/current.log', NULL, NULL, $offset)) {
					
					if ($file_content = strstr($file_content, "\n")) {
						
						if ($fp = fopen( NGINXM_ABSPATH .'log/current.log', "w+" )) {
							fwrite($fp, $file_content);
							fclose($fp);
						}
					}
				}
			}
		}
		
		function purgeImageOnEdit($attachment_id) {
			
			$this->log( "Purging media on edit BEGIN ===" );
			
			// check if is an image
			if ( wp_attachment_is_image( $attachment_id ) ) {
				
				// 1. purge image "main" URL
				$this->purgeUrl( wp_get_attachment_url( $attachment_id ), false );
				
				$attachment = wp_get_attachment_metadata( $attachment_id );
				
				// 2. purge image other sizes
				if ( $attachment['sizes'] ) {
					foreach ( $attachment['sizes'] as $size_name => $size ) {
						
						$resize_image = wp_get_attachment_image_src( $attachment_id, $size_name );
						if ( $resize_image )
							$this->purgeUrl( $resize_image[0], false );
					}
				}
				
			} else {
				$this->log( "Media (id $attachment_id) edited: no image", "WARNING" );
			}
			
			$this->log( "Purging media on edit END ^^^" );
			
		}
		
	}
	
}